query status($input: ID!) {
    detectionEntitiesUploadStatus(receiptId: $input) {
        status
        error
        result {
            dataModels {
                ...UploadStatisticDetails
            }
            globalHelpers {
                ...UploadStatisticDetails
            }
            lookupTables {
                ...UploadStatisticDetails
            }
            policies {
                ...UploadStatisticDetails
            }
            rules {
                ...UploadStatisticDetails
            }
            queries {
                ...UploadStatisticDetails
            }
            correlationRules {
                ...UploadStatisticDetails
            }
        }
    }
}

fragment UploadStatisticDetails on UploadStatistics {
    modified
    new
    total
    deleted
}